#!/usr/bin/python 2.7

import time
import json
from ESL_Structure import *

exeCmdType = 2

class CmdHandler:
	def __init__(self, _mqttClient, _publichTopic):
		self.mqttClient = _mqttClient
		self.publichTopic = _publichTopic
		self.Cmddict = dict()
		self.structure = ESLStructure()
		#self.structure.traversal()
		#self.addCmd(2,0x106,self.cbGetpermitTagList)

		#Gateway
		self.addCmd(2,0x108,self.cbZbReboot)
		self.addCmd(2,0x109,self.cbgetRouterList)
		self.addCmd(2,0x111,self.cbGetFtpInfo) #273

		#Router 
		self.addCmd(2,0x206,self.cbgetTagList) #518
		self.addCmd(2,0x210,self.cbRouterSetTxPower) #528 
		self.addCmd(2,0x211,self.cbRouterGetTxPower) #529 

		#Tag
		self.addCmd(2,0x301,self.cbTagSetUpdatePeriod) #769
		self.addCmd(2,0x302,self.cbTagGetUpdatePeriod) #770
		self.addCmd(2,0x303,self.cbTagSetTimeout) #771 
		self.addCmd(2,0x304,self.cbTagGetTimeout) #772
		self.addCmd(2,0x307,self.cbTagImageUpdate) #775
		self.addCmd(2,0x309,self.cbTagImageRefresh) #777

	def addCmd(self, _cmdType, _cmdId, callBack):
		if (_cmdType == exeCmdType):
			self.Cmddict[_cmdId] = callBack

	def parseCmd(self, msg):
		jcmd = json.loads(msg)
		if (jcmd['Cmd-Type'] == exeCmdType):
			if (jcmd['Cmd-Id'] in self.Cmddict):
				#Action
				self.Cmddict[jcmd['Cmd-Id']](jcmd)
			else:
				print "'Cmd-Id error"
				jcmd['Code'] = 1
				self.mqttClient.publish(self.publichTopic, json.dumps(jcmd))
		else:
			print "Cmd-Type error"

	def genJsonResponse(self, _id, cmdId):
			jsonRes = json.loads('{"Cmd-Type": 3}')
			jsonRes['Cmd-Type']=3
			jsonRes['Transaction-Id']=_id
			jsonRes['Cmd-Id']=cmdId
			jsonRes['Code']=0
			jsonRes['Parameter']=json.loads("{}")
			return jsonRes



	def emptyCmdhandler(self, *args):
		print "Not support this command"

	def cbZbReboot(self, jsonCmd):  #0x108  264 zd-reboot
		print "[GetCmd] Get zd-reboot cmd"
		jsonCmd['Code']=0
		self.mqttClient.publish(self.publichTopic, json.dumps(jsonCmd))

	def cbgetRouterList(self, jsonCmd):  #0x109  265 zd-get-router-list
		print "[GetCmd] zd-get-router-list cmd"
		res = self.genJsonResponse(jsonCmd['Transaction-Id'], jsonCmd['Cmd-Id'])
		res['Parameter']['device-ieeeadr-index']=0
		res['Parameter']['device-ieeeadr-count']=len(self.structure.getRouters())
		res['Parameter']['device-ieeeadr-list']=json.loads("{}")
		res['Parameter']['device-ieeeadr-list']=self.structure.getRouters().keys()
		#print json.dumps(res)
		self.mqttClient.publish(self.publichTopic, json.dumps(res))

	def cbGetFtpInfo(self, jsonCmd):  #0x111  273 get-ftp-info
		print "[GetCmd] get-ftp-info cmd"
		res = self.genJsonResponse(jsonCmd['Transaction-Id'], jsonCmd['Cmd-Id'])
		res['Parameter']['username']="ftpName"
		res['Parameter']['password']="123456"
		self.mqttClient.publish(self.publichTopic, json.dumps(res))

	def cbgetTagList(self, jsonCmd):  #0x206 518 get-tag-list
		print "[GetCmd] get-tag-list cmd"
		addr=jsonCmd['Parameter']['router-addr']
		res = self.genJsonResponse(jsonCmd['Transaction-Id'], jsonCmd['Cmd-Id'])
		if addr in self.structure.getRouters():
			res['Parameter']['router-addr']=jsonCmd['Parameter']['router-addr']
			res['Parameter']['device-ieeeadr-index']=0
			res['Parameter']['device-ieeeadr-count']=len(self.structure.getRouters()[addr].getTags())
			res['Parameter']['device-ieeeadr-list']=json.loads("{}")
			res['Parameter']['device-ieeeadr-list']=self.structure.getRouters()[addr].getTags().keys()
		else:
			res['Code']=2 #Router not find
		#print json.dumps(res)
		self.mqttClient.publish(self.publichTopic, json.dumps(res))

	def cbRouterSetTxPower(self, jsonCmd):  #0x210 528 set-zd-tx-power
		print "[GetCmd] set-zd-tx-power cmd"
		addr=jsonCmd['Parameter']['router-addr']
		if addr in self.structure.getRouters():
			self.structure.getRouters()[addr].setTxPower(jsonCmd['Parameter']['tx-level-R'],jsonCmd['Parameter']['tx-level-T'])
			jsonCmd['Code']=0
			#res['Parameter']['target-addr']=jsonCmd['Parameter']['router-addr']
			#res['Parameter']['tx-level-R']=0
			#res['Parameter']['tx-level-T']=len(self.structure.getRouters()[addr].getTags())
		else:
			jsonCmd['Code']=2 #Router not find
		#print json.dumps(res)
		self.mqttClient.publish(self.publichTopic, json.dumps(jsonCmd))

	def cbRouterGetTxPower(self, jsonCmd):  #0x211 529 get-zd-tx-power
		print "[GetCmd] get-zd-tx-power cmd"
		addr=jsonCmd['Parameter']['router-addr']
		res = self.genJsonResponse(jsonCmd['Transaction-Id'], jsonCmd['Cmd-Id'])
		if addr in self.structure.getRouters():
			powerlist = self.structure.getRouters()[addr].getTxPower()
			res['Parameter']['router-addr']=jsonCmd['Parameter']['router-addr']
			res['Parameter']['tx-level-R']=powerlist[0]
			res['Parameter']['tx-level-T']=powerlist[1]
		else:
			res['Code']=2 #Router not find
		self.mqttClient.publish(self.publichTopic, json.dumps(res))

	def cbTagSetUpdatePeriod(self, jsonCmd):  #0x301 769 SetUpdatePeriod
		print "[GetCmd] SetUpdatePeriod cmd"
		addr=jsonCmd['Parameter']['tag-addr']
		if addr in self.structure.getTags():
			self.structure.getTags()[addr].setPeriod(jsonCmd['Parameter']['data-request-period'],jsonCmd['Parameter']['state-report-period'])
			jsonCmd['Code']=0
		else:
			jsonCmd['Code']=3 #tag not find
		self.mqttClient.publish(self.publichTopic, json.dumps(jsonCmd))

	def cbTagGetUpdatePeriod(self, jsonCmd):  #0x302 770 GetUpdatePeriod
		print "[GetCmd] GetUpdatePeriod cmd"
		addr=jsonCmd['Parameter']['tag-addr']
		res = self.genJsonResponse(jsonCmd['Transaction-Id'], jsonCmd['Cmd-Id'])
		if addr in self.structure.getTags():
			periodlist = self.structure.getTags()[addr].getPeriod()
			res['Parameter']['tag-addr']=jsonCmd['Parameter']['tag-addr']
			res['Parameter']['data-request-period']=periodlist[0]
			res['Parameter']['state-report-period']=periodlist[1]
		else:
			res['Code']=3 #tag not find
		self.mqttClient.publish(self.publichTopic, json.dumps(res))

	def cbTagSetTimeout(self, jsonCmd):  #0x303 771 TagSetTimeout
		print "[GetCmd] TagSetTimeout cmd"
		addr=jsonCmd['Parameter']['tag-addr']
		if addr in self.structure.getTags():
			self.structure.getTags()[addr].setTimeout(jsonCmd['Parameter']['timeout'])
			jsonCmd['Code']=0
		else:
			jsonCmd['Code']=3 #tag not find
		self.mqttClient.publish(self.publichTopic, json.dumps(jsonCmd))

	def cbTagGetTimeout(self, jsonCmd):  #0x304 772 TagGetTimeout
		print "[GetCmd] TagGetTimeout cmd"
		addr=jsonCmd['Parameter']['tag-addr']
		res = self.genJsonResponse(jsonCmd['Transaction-Id'], jsonCmd['Cmd-Id'])
		if addr in self.structure.getTags():
			timeout = self.structure.getTags()[addr].getTimeout()
			res['Parameter']['tag-addr']=jsonCmd['Parameter']['tag-addr']
			res['Parameter']['timeout']=timeout
		else:
			res['Code']=3 #tag not find
		self.mqttClient.publish(self.publichTopic, json.dumps(res))

	def cbTagImageUpdate(self, jsonCmd):  #0x307 775 image-update 
		print "[GetCmd] Tag image-update cmd"
		addr=jsonCmd['Parameter']['tag-addr']
		res = self.genJsonResponse(jsonCmd['Transaction-Id'], jsonCmd['Cmd-Id'])
		if addr in self.structure.getTags():
			jsonCmd['Code']=0
		else:
			jsonCmd['Code']=3 #tag not find
		#self.mqttClient.publish(self.publichTopic, json.dumps(jsonCmd)) #ack
		time.sleep(5) #delay 5 sec
		if addr in self.structure.getTags():
			res['Parameter']['image-name']=jsonCmd['Parameter']['image-name'] 
			res['Parameter']['image-crc']=jsonCmd['Parameter']['image-crc']
			self.mqttClient.publish(self.publichTopic, json.dumps(res))

	def cbTagImageRefresh(self, jsonCmd):  #0x309 777 ImageRefresh
		print "[GetCmd] Tag Image Refresh cmd"
		addr=jsonCmd['Parameter']['tag-addr']
		res = self.genJsonResponse(jsonCmd['Transaction-Id'], jsonCmd['Cmd-Id'])
		if addr in self.structure.getTags():
			jsonCmd['Code']=0
		else:
			jsonCmd['Code']=3 #tag not find
		#self.mqttClient.publish(self.publichTopic, json.dumps(jsonCmd)) #ack

		delayTime = jsonCmd['Parameter']['timedelay']
		time.sleep(delayTime) #delay sec
		if addr in self.structure.getTags():
			res['Parameter']['image-crc']=jsonCmd['Parameter']['image-crc']
			res['Parameter']['total-image-crc']="2A3B"
			self.mqttClient.publish(self.publichTopic, json.dumps(res))

	def cbGetpermitTagList(self, jsonCmd):  #0x106
		print "success"
		for dev in jsonCmd['Parameter']['device-ieeeadr-list']:
			print dev